package net.bigeon.gclc.socket;

/*-
 * #%L
 * GCLC Socket
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;

import net.bigeon.gclc.ConsoleApplication;

/** A test server
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({ "javadoc", "nls" })
public class TestServer {

    public static final int                      PORT = 12345;
    private static SocketConsoleApplicationShell SHELL;
    private static ConnexionManager<Socket>      manager;

    private static Thread                 server;
    private static PluggableConsoleInput  input;
    private static PluggableConsoleOutput output;

    public static synchronized void closeServer() {
        SHELL.stop();
        input.close();
        output.close();
        SHELL = null;
        server = null;
    }

    public static synchronized Thread getServer() throws IOException {
        if (server == null) {
            server = new Thread(getShell(), "gclcServer");
            server.start();
        }
        return server;
    }

    private static SocketConsoleApplicationShell getShell() throws UnknownHostException {
        if (SHELL == null) {
            input = new PluggableConsoleInput();
            input.setPrompt("> \n");
            output = new PluggableConsoleOutput();
            manager = new DConnexionManager<>();
            SHELL = new SocketConsoleApplicationShell(PORT, InetAddress.getLocalHost());
            final ConsoleApplication app = ConsoleTestApplication.create(output, input,
                    manager);
            SHELL.setInterface(new SocketConsoleInterface(input, output));
            SHELL.setConnexionManager(manager);
            SHELL.setApplication(app);
            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    app.start();
                }
            });
            th.start();
            try {
                final Object waiting = new Object();
                synchronized (waiting) {
                    while (!input.isPrompting()) {
                        waiting.wait(50);
                    }
                }
            } catch (final InterruptedException e) {
                Thread.currentThread().interrupt();
            }
        }
        return SHELL;
    }

    /** @param args no argument
     * @throws IOException if the server starting failed */
    public static void main(final String... args) throws IOException {
        try {
            getServer().join();
        } catch (final InterruptedException e) {
            e.printStackTrace();
        }
    }

}
